function [origlen, lenaftersample, raw, data] = ...
    motionloader(Filelist, down_sample_rate)
%MOTIONLOADER load in all motions into the matrix format
%   Filelist: a list of files
%   down_sample_rate: # frame skips in downsampling (using 1 to keep the original frame rate)
%   RETURN:
%   raw: the 3-dimensional matrix of all mocap sequences
%   data: the mocap data  (in a 2-dimensional matrix)
%   len: the length for each mocap sequence
%% Initialization
if (nargin < 2)
    down_sample_rate = 1;
end;
l=length(Filelist);
len=zeros(1, l);
point=0;
datapca=[];

%% Read in all mocap sequences, and downsample them
for i=1:l
    data1=[];
    disp(Filelist{i});
    [pathstr, name, ext] = fileparts(Filelist{i});
    % amc to matrix, each row of matrix reprensent one frame
    if ( strcmp(ext, '.amc') ==1 )
        data1=amc_to_matrix(Filelist{i});
    elseif ( strcmp(ext, '.txt') ==1 )
        data1=read_amctxt(Filelist{i});
        amcfilename=sprintf('./result/%s.amc', Filelist{i});
        matrix_to_amc(amcfilename, data1);
    else
        continue;
    end

    [x,y]=size(data1);

    origlen(i) = x;
    lenaftersample(i)=x;
    lenaftersample(i) = floor(lenaftersample(i)/down_sample_rate);
    for j=1:lenaftersample(i)
        data(point+j,1:y)=data1((j*down_sample_rate-down_sample_rate +1),1:y);
        raw(j, 1:y, i) = data1((j*down_sample_rate-down_sample_rate +1), 1:y);
    end;
    point=point+lenaftersample(i);
end;

end
