% pcasvd: Perform PCA using SVD.

% Cheng Ren / ??
% Beihang University
% April 2011

function [signals,V, S, SS] = pcasvd(data, n, compressiontheshold)
% data - MxN matrix of input data
% (M dimensions, N trials)
% n - the number of dimension for the reduced data set
% compressiontheshold - optional, when this is set, n will be decided automatically, 
% reference paper: 'K. Fukunaga. Statistical Pattern Recognition, 2nd Edition. John Hopkins University Press, Baltimore, 1989.'
% signals - nxN matrix of projected data
% V - each column is a PC
% SS - Mx1 matrix of variances

[M,N] = size(data);

mn = mean(data,2);
data = data - repmat(mn,1,N);  % subtract off the mean for each dimension
Y = data' / sqrt(N-1); % construct the matrix Y
[u,S,V] = svd(Y,0);  % SVD does it all
Sd = diag(S);  % calculate the variances
SS = Sd .* Sd;

if (nargin < 2)
    n = M;
end

if (nargin > 2)
    sigmasqr = sum(SS);

    sigmapartialsqr = 0;
    for i=1:M
        sigmapartialsqr = sigmapartialsqr + SS(i);
        if(sigmapartialsqr >= compressiontheshold * sigmasqr)
            n = i;
            break;
        end
    end
end

% project the original data
signals = V(:,1:n)' * data;





