% Behavior semgmentation using PPCA method
% reference paper : Barbic, J., et al. Segmenting Motion Capture Data into
% Distinct Behaviors. in Proceedings of Graphics Interface Conference.
% 2004.

% Cheng Ren / ??
% Beihang University
% August 2011

%% All motion capture sequences to be processed
%list = dir('./ExperimentCMUData/*.amc');
list = dir('./ExperimentCMUData/');
filenum = length(list);
for i=1:filenum
    file{i} = list(i).name;
end

[origlen, lenaftersample, rawdata, amcdata] = motionloader(file, 1);
totalframes = size(amcdata,1);

%% settings according to the paper
shortestbehlen = 300;
T = shortestbehlen / 2;
delt = 10;
threshold = 500000;
allcuts = [];

%% foward pass
disp('doing fowrad pass...');
direction = 1;    %% 1 corresponds to forward pass, -1 corresponds to backward pass
startframe = 1;
endframe = totalframes;
allH = [];
lastcut = startframe;
cutsInFP = lastcut;   % cuts in forward pass
K = T;
while (K <= endframe-shortestbehlen)

    % compute PCA and SVD
    [signals,V, S, SS] = pcasvd( amcdata(lastcut:direction:K,7:62)', 0, 0.95 );  % tau = 0.95
    r = size(signals, 1);

    % compute equation (7)
    temp = SS(r+1 : 56);
    avgdiscard = (sum(temp))/(56-r);

    % compute covariance matrix, equation (9)
    SS2 = S;
    for i=1 : r
        SS2(i,i) = SS(i);
    end
    for i=r+1 : 56
        SS2(i,i) = avgdiscard;
    end
    n = abs(K - lastcut) + 1;
    C = (V*SS2*V')/(n-1);           %% covariance matrix
    xbar = sum(amcdata(lastcut:direction:K,7:62)) / n;        %% center of the motion

    % compute H, equation (10)
    invc = inv(C);
    temp = 0;
    for i = K+1*direction :direction : K+T*direction 
        xdist = amcdata(i,7:62) - xbar;
        temp = temp + xdist * invc * xdist';
    end
    H = temp / T;                   %% Mahalanobis distance
    allH = [allH, H];

    K = K + direction * delt;

    % locate the cut postition
    lengthallH = length(allH);
    if (lengthallH > T/delt)

        [pks,locs] = findpeaks(allH, 'MINPEAKDISTANCE', delt, 'SORTSTR', 'descend');
        %[msg id] = lastwarn;  warning('off',id)      %% turning off the
        %peaks warning

        nloc = length(locs);
        if ( nloc>0 && locs(1)>T/delt/2 && locs(1)<(lengthallH-T/delt/2) )
            vallyspan = allH(1:locs(1));     %% valley usually exisits before  a peak
            valley = min(vallyspan);
            if ( pks(1) - valley > threshold )     %% a cut is found at the peak
                lastcut = lastcut + direction * (T - 1 + (locs(1) -1) * delt );
                K = lastcut + direction * (T  - 1);
                cutsInFP = [cutsInFP, lastcut];
%                             figure;
%                             plot(allH); hold on;
%                             plot(locs(1),pks(1),'k^','markerfacecolor',[1 0 0]);
                allH = [];
            end
        end
    end

end
cutsInFP = [cutsInFP, endframe];
cutsInFP
allcuts = [allcuts, cutsInFP];

%% backward pass
disp('doing backward pass...');
direction = -1;
lenFP = length(cutsInFP);
for m=lenFP: (-1) :2               %% do the backward PPCA for each segement resulted from forward pass
    
    startframe = cutsInFP(m-1);
    endframe = cutsInFP(m);
    allH = [];
    lastcut = endframe;
    cutsInBP = [];   % cuts in backward pass
    K = endframe - T + 1;

    while (K >= startframe + shortestbehlen - 1)

        %% now the below process is the same with that in the forward pass
        % compute PCA and SVD
        [signals,V, S, SS] = pcasvd(amcdata(lastcut:direction:K,7:62)',0,0.95);           %%%%%%backward
        r = size(signals, 1);

        % compute equation (7)
        temp = SS(r+1 : 56);
        avgdiscard = (sum(temp))/(56-r);

        % compute covariance matrix, equation (9)
        SS2 = S;
        for i=1 : r
            SS2(i,i) = SS(i);
        end
        for i=r+1 : 56
            SS2(i,i) = avgdiscard;
        end
        n = abs(K - lastcut) + 1;
        C = (V*SS2*V')/(n-1);
        xbar = sum(amcdata(lastcut:direction:K,7:62)) / n;  %%%%backward

        % compute H, equation (10)
        invc = inv(C);
        temp = 0;
        for i = K+1*direction :direction : K+T*direction       %%%%backward
            xdist = amcdata(i,7:62) - xbar;
            temp = temp + xdist * invc * xdist';
        end
        H = temp / T;
        allH = [allH, H];

        K = K + direction * delt;                         %%%%backward

        % locate the cut postition
        lengthallH = length(allH);
        if (lengthallH > T/delt)

            [pks,locs] = findpeaks(allH, 'MINPEAKDISTANCE', delt, 'SORTSTR', 'descend');
            nloc = length(locs);

            if ( nloc>0 && locs(1)>T/delt/2 && locs(1)<(lengthallH-T/delt/2) )
                vallyspan = allH(1:locs(1));     %% valley usually exisits before  a peak
                valley = min(vallyspan);
                if ( pks(1) - valley > threshold )     %% a cut is found at the peak
                    lastcut = lastcut + direction *( (T - 1) +  (locs(1) -1) * delt ); %%%%backward
                    K = lastcut + direction * (T-1);              %%%%backward
                    cutsInBP = [cutsInBP, lastcut];
                    %                             figure;
                    %                             plot(allH); hold on;
                    %                             plot(locs(1),pks(1),'k^','markerfacecolor',[1 0 0]);
                    allH = [];
                end
            end
        end

    end
    allcuts = [allcuts, cutsInBP];

end

disp('results:');
allcuts = sort(allcuts)

%% output the segments
lenallcuts = length(allcuts);
for i=1:lenallcuts-1
    amcfilename=sprintf('./result/seg_%d.amc', i);
    matrix_to_amc(amcfilename, amcdata(allcuts(i):allcuts(i+1)-1,:) );
end
